'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/*
Copyright (c) 2016, Marco Sampellegrini <babbonatale@alpacaaa.net>

Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee
is hereby granted, provided that the above copyright notice and this permission notice appear in all copies.
THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE
INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE
FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

var Dimensions = require('react-native').Dimensions;

var computeStyles = function computeStyles(dimensions, base) {
  for (var _len = arguments.length, extra = Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    extra[_key - 2] = arguments[_key];
  }

  return extra.reduce(function (acc, fn) {
    var properties = fn(dimensions);
    var merged = Object.keys(properties).reduce(function (s_acc, key) {
      return _extends({}, s_acc, _defineProperty({}, key, _extends({}, base[key], properties[key])));
    }, {});

    return _extends({}, acc, merged);
  }, base);
};

var getStyles = function getStyles(base) {
  var styles = {};
  var computed = {};

  Object.keys(base).map(function (key) {
    Object.defineProperty(styles, key, {
      get: function get() {
        return computed[key];
      }
    });
  });

  var update = function update(newStyles) {
    if (JSON.stringify(newStyles) === JSON.stringify(computed)) {
      return false;
    }

    computed = newStyles;
    return true;
  };

  return { styles: styles, update: update };
};

var createStyles = exports.createStyles = function createStyles(base) {
  for (var _len2 = arguments.length, extra = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
    extra[_key2 - 1] = arguments[_key2];
  }

  var _getStyles = getStyles(base);

  var styles = _getStyles.styles;
  var update = _getStyles.update;


  styles.onLayout = function (fn) {
    return function (event) {
      var dimensions = Dimensions.get('window');
      var computed = computeStyles.apply(undefined, [dimensions, base].concat(extra));
      var changed = update(computed);
      if (changed && fn) fn();
    };
  };

  styles.onLayout()();
  return styles;
};

var createQueryFn = function createQueryFn(test) {
  return function (target, styles) {
    return function (dimensions) {
      if (!test(target, dimensions)) return {};

      return typeof styles === 'function' ? styles(dimensions) : styles;
    };
  };
};

var maxHeight = exports.maxHeight = createQueryFn(function (target, _ref) {
  var height = _ref.height;
  return target >= height;
});
var minHeight = exports.minHeight = createQueryFn(function (target, _ref2) {
  var height = _ref2.height;
  return target <= height;
});

var maxWidth = exports.maxWidth = createQueryFn(function (target, _ref3) {
  var width = _ref3.width;
  return target >= width;
});
var minWidth = exports.minWidth = createQueryFn(function (target, _ref4) {
  var width = _ref4.width;
  return target <= width;
});

var aspectRatioRound = function aspectRatioRound(value) {
  return value.toFixed(2);
};
var aspectRatioQuery = function aspectRatioQuery(fn) {
  return createQueryFn(function (target, _ref5) {
    var width = _ref5.width;
    var height = _ref5.height;

    return fn(aspectRatioRound(target), aspectRatioRound(width / height));
  });
};

var maxAspectRatio = exports.maxAspectRatio = aspectRatioQuery(function (target, aspectRatio) {
  return target >= aspectRatio;
});
var minAspectRatio = exports.minAspectRatio = aspectRatioQuery(function (target, aspectRatio) {
  return target <= aspectRatio;
});
var aspectRatio = exports.aspectRatio = aspectRatioQuery(function (target, aspectRatio) {
  return target == aspectRatio;
});
